<?php

if (!defined('ABSPATH')) exit;


if (!class_exists('ElfsightFacebookFeedApi')) {
	class ElfsightFacebookFeedApi extends ElfsightFacebookFeedApiCore {
		private $routes = array(
			'' => 'requestController'
		);

		private $tokensList = array(
			'395202813876688|73e8ede72008b231a0322e40f0072fe6',
			'177899259380474|JgdlhmU-J0dq55bKng0xywDIgIo',
			'128947497630881|rlgLr8wtMUWRw7hbcLcsgoa01-k',
			'1298990730176646|rU4QhoOaYPWQng6-k_QdxRoVNaA',
			'1693018934313805|VYDjx69NpsgkDEAm79cYD0fxJBk',
			'257106408010811|MwCorr7qsyIeU_GjdPFIEw3-_P8',
			'386881278380301|NW_PiECD9TLVe0UNMsB5H9HkPEo',
			'277269689412168|o3k5mzmHsT-prqc2qizxk_DoXEQ',
			'1439406912938596|4JDCXisJ2d-1EWeObBt27DybH5c',
			'927474184059774|ekzzwIV9JnvG-ELYWut9wIvf4Y0',
		);

		private $usageStoreTime = 86400;

		public $tokensTableName;

		public $tokens;
		public $token;

		public function __construct($config) {
			parent::__construct($config, $this->routes);

			$this->tokensTableName = $this->helper->getTableName('tokens');

			if (!$this->helper->tableExist('tokens')) {
				$this->createTokensTable();
				$this->seedTokensTable();
			}

			$this->setCurrentToken();

			register_deactivation_hook($this->pluginFile, array($this, 'dropTokensTable'));
		}

		public function setCurrentToken() {
			$this->tokens = $this->getTokens();
			$this->token = $this->tokens[0];
		}

		public function requestController() {
			$q = $this->input('q');

			$cache_key = $this->cache->key($q, array('access_token', 'fields'));
			$data = $this->cache->get($cache_key);

			if (empty($data)) {
				if ($this->isRateLimited($this->token)) {
					return $this->error(429, 'facebook rate limiting');
				}

				$request_url = $this->buildRequestUrl($q);

				$response = $this->request('GET', $request_url);

				if (!empty($response)) {
					if (!empty($response['headers']) && !empty($response['headers']['x-app-usage'])) {
						$app_usage = json_decode($response['headers']['x-app-usage'], true);

						$this->updateToken($this->token, $app_usage);

						if ($this->isRateLimited($this->token)) {
							$this->setCurrentToken();
							return $this->requestController();
						}
					}

					if (!empty($response['body'])) {
						$data = $response['body'];
					}

					if (!empty($response['http_code']) && $response['http_code'] === '200') {
						$this->cache->set($cache_key, $data);
					}

				} else {
					return $this->error();
				}
			}

			if (empty($data)) {
				$data = $this->cache->get($cache_key, false);
			}

			return $this->response($data, true);
		}

		public function buildRequestUrl(&$url) {
			$url = $this->helper->removeQueryParam($url, 'access_token');
			$url = $this->helper->addQueryParam($url, 'access_token', $this->token['token']);
			$url = $this->helper->addQueryParam($url, 'locale', 'en_US');

			if (stripos($url, 'https://graph.facebook.com') === false) {
				$url = 'https://graph.facebook.com/' . $url;
			}

			return $url;
		}

		private function isRateLimited($token) {
			return !empty($token['usage_value']) && $token['usage_value'] > 90;
		}

		private function getTokens($token = false) {
			global $wpdb;

			$result = array();

			$select_sql = 'SELECT * FROM `' . esc_sql($this->tokensTableName) . '`';
			if ($token) {
				$select_sql .= ' WHERE `token` = "' . esc_sql($token) . '"';
			}

			$list = $wpdb->get_results($select_sql, ARRAY_A);

			foreach ($list as &$token) {
				if ($token['updated_at'] + $this->usageStoreTime < time()) {
					$token['usage_value'] = 0;
					$token['usage_data'] = '{}';
				}
				$result[] = $token;
			}

			usort($result, array($this, 'sortTokens'));

			return $result;
		}

		private function sortTokens($a, $b) {
			return $a['usage_value'] > $b['usage_value'];
		}

		private function updateToken($token, $app_usage) {
			global $wpdb;

			if (empty($token) || empty($app_usage)) {
				return false;
			}

			$usage_value = 0;
			foreach ($app_usage as $usage_param) {
				if ($usage_param > $usage_value) {
					$usage_value = $usage_param;
				}
			}

			$this->token['app_usage'] = $app_usage;
			$this->token['usage_value'] = $usage_value;

			return !!$wpdb->update(
				$this->tokensTableName,
				array(
					'token' => $token['token'],
					'usage_data' => json_encode($app_usage),
					'usage_value' => $usage_value,
					'updated_at' => time()
				),
				array('id' => $token['id'])
			);
		}

		private function createTokensTable() {
			if (!function_exists('dbDelta')) {
				require(ABSPATH . 'wp-admin/includes/upgrade.php');
			}

			dbDelta(
				'CREATE TABLE `' . esc_sql($this->tokensTableName) . '` (
                    `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                    `token` varchar(255) NOT NULL,
                    `usage_data` text NOT NULL,
                    `usage_value` int(3) NOT NULL,
                    `updated_at` int(10) NOT NULL,
                    PRIMARY KEY (`id`)
                ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;'
			);
		}

		private function seedTokensTable() {
			global $wpdb;

			foreach ($this->tokensList as $token) {
				$wpdb->insert($this->tokensTableName, array(
					'token' => $token,
					'usage_data' => '{}',
					'usage_value' => 0,
					'updated_at' => 0
				));
			}
		}

		public function dropTokensTable() {
			global $wpdb;

			return $wpdb->query('DROP TABLE IF EXISTS `' . $this->tokensTableName . '`');
		}
	}
}